# This file was auto-generated by Fern from our API Definition.

from .letta_streaming_response import LettaStreamingResponse
from .message_update_content import MessageUpdateContent
from .messages_list_response import MessagesListResponse

__all__ = ["LettaStreamingResponse", "MessageUpdateContent", "MessagesListResponse"]



# This file was auto-generated by Fern from our API Definition.

from .types import LettaStreamingResponse, MessageUpdateContent, MessagesListResponse

__all__ = ["LettaStreamingResponse", "MessageUpdateContent", "MessagesListResponse"]



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class ActionResponseModel(UncheckedBaseModel):
    """
    Action response data model.
    """

    properties: typing.Dict[str, typing.Optional[typing.Any]]
    title: str
    type: str
    required: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .agents_search_request_combinator import AgentsSearchRequestCombinator
from .agents_search_request_search_item import AgentsSearchRequestSearchItem
from .agents_search_request_search_item_direction import AgentsSearchRequestSearchItemDirection
from .agents_search_request_search_item_direction_direction import AgentsSearchRequestSearchItemDirectionDirection
from .agents_search_request_search_item_direction_value import AgentsSearchRequestSearchItemDirectionValue
from .agents_search_request_search_item_one import AgentsSearchRequestSearchItemOne
from .agents_search_request_search_item_one_operator import AgentsSearchRequestSearchItemOneOperator
from .agents_search_request_search_item_two import AgentsSearchRequestSearchItemTwo
from .agents_search_request_search_item_zero import AgentsSearchRequestSearchItemZero
from .create_agent_request_tool_rules_item import CreateAgentRequestToolRulesItem
from .update_agent_tool_rules_item import UpdateAgentToolRulesItem

__all__ = [
    "AgentsSearchRequestCombinator",
    "AgentsSearchRequestSearchItem",
    "AgentsSearchRequestSearchItemDirection",
    "AgentsSearchRequestSearchItemDirectionDirection",
    "AgentsSearchRequestSearchItemDirectionValue",
    "AgentsSearchRequestSearchItemOne",
    "AgentsSearchRequestSearchItemOneOperator",
    "AgentsSearchRequestSearchItemTwo",
    "AgentsSearchRequestSearchItemZero",
    "CreateAgentRequestToolRulesItem",
    "UpdateAgentToolRulesItem",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AgentsSearchRequestCombinator,
    AgentsSearchRequestSearchItem,
    AgentsSearchRequestSearchItemDirection,
    AgentsSearchRequestSearchItemDirectionDirection,
    AgentsSearchRequestSearchItemDirectionValue,
    AgentsSearchRequestSearchItemOne,
    AgentsSearchRequestSearchItemOneOperator,
    AgentsSearchRequestSearchItemTwo,
    AgentsSearchRequestSearchItemZero,
    CreateAgentRequestToolRulesItem,
    UpdateAgentToolRulesItem,
)
from . import archival_memory, context, core_memory, memory_variables, messages, sources, templates, tools
from .memory_variables import MemoryVariablesListResponse
from .messages import LettaStreamingResponse, MessageUpdateContent, MessagesListResponse
from .templates import TemplatesMigrateResponse

__all__ = [
    "AgentsSearchRequestCombinator",
    "AgentsSearchRequestSearchItem",
    "AgentsSearchRequestSearchItemDirection",
    "AgentsSearchRequestSearchItemDirectionDirection",
    "AgentsSearchRequestSearchItemDirectionValue",
    "AgentsSearchRequestSearchItemOne",
    "AgentsSearchRequestSearchItemOneOperator",
    "AgentsSearchRequestSearchItemTwo",
    "AgentsSearchRequestSearchItemZero",
    "CreateAgentRequestToolRulesItem",
    "LettaStreamingResponse",
    "MemoryVariablesListResponse",
    "MessageUpdateContent",
    "MessagesListResponse",
    "TemplatesMigrateResponse",
    "UpdateAgentToolRulesItem",
    "archival_memory",
    "context",
    "core_memory",
    "memory_variables",
    "messages",
    "sources",
    "templates",
    "tools",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ActionModel,
    ActionParametersModel,
    ActionResponseModel,
    AgentEnvironmentVariable,
    AgentState,
    AgentStateToolRulesItem,
    AgentType,
    AppAuthScheme,
    AppAuthSchemeAuthMode,
    AppModel,
    AssistantMessage,
    AssistantMessageContent,
    AuthRequest,
    AuthResponse,
    AuthSchemeField,
    Block,
    BlockUpdate,
    ChatCompletionMessageToolCall,
    ChildToolRule,
    ConditionalToolRule,
    ConflictErrorBody,
    ContextWindowOverview,
    CreateBlock,
    E2BSandboxConfig,
    EmbeddingConfig,
    EmbeddingConfigEmbeddingEndpointType,
    FileMetadata,
    Function,
    FunctionDefinition,
    FunctionTool,
    Health,
    HttpValidationError,
    InitToolRule,
    InternalServerErrorBody,
    Job,
    JobStatus,
    JobType,
    LettaMessageUnion,
    LettaRequest,
    LettaRequestConfig,
    LettaResponse,
    LettaUsageStatistics,
    LlmConfig,
    LlmConfigModelEndpointType,
    LocalSandboxConfig,
    Memory,
    Message,
    MessageCreate,
    MessageCreateContent,
    MessageCreateRole,
    MessageRole,
    NotFoundErrorBody,
    NotFoundErrorBodyMessage,
    Organization,
    OrganizationCreate,
    Passage,
    Provider,
    ReasoningMessage,
    Run,
    SandboxConfig,
    SandboxConfigCreate,
    SandboxConfigCreateConfig,
    SandboxConfigUpdate,
    SandboxConfigUpdateConfig,
    SandboxEnvironmentVariable,
    SandboxEnvironmentVariableCreate,
    SandboxEnvironmentVariableUpdate,
    SandboxType,
    Source,
    SystemMessage,
    SystemMessageContent,
    TerminalToolRule,
    TextContent,
    Tool,
    ToolCall,
    ToolCallDelta,
    ToolCallMessage,
    ToolCallMessageToolCall,
    ToolCreate,
    ToolReturnMessage,
    ToolReturnMessageStatus,
    ToolRuleType,
    ToolType,
    UsageStatistics,
    User,
    UserCreate,
    UserMessage,
    UserMessageContent,
    UserUpdate,
    ValidationError,
    ValidationErrorLocItem,
)
from .errors import ConflictError, InternalServerError, NotFoundError, UnprocessableEntityError
from . import agents, blocks, health, jobs, models, providers, runs, sources, tag, tools
from .agents import (
    AgentsSearchRequestCombinator,
    AgentsSearchRequestSearchItem,
    AgentsSearchRequestSearchItemDirection,
    AgentsSearchRequestSearchItemDirectionDirection,
    AgentsSearchRequestSearchItemDirectionValue,
    AgentsSearchRequestSearchItemOne,
    AgentsSearchRequestSearchItemOneOperator,
    AgentsSearchRequestSearchItemTwo,
    AgentsSearchRequestSearchItemZero,
    CreateAgentRequestToolRulesItem,
    UpdateAgentToolRulesItem,
)
from .client import AsyncLetta, Letta
from .environment import LettaEnvironment
from .version import __version__

__all__ = [
    "ActionModel",
    "ActionParametersModel",
    "ActionResponseModel",
    "AgentEnvironmentVariable",
    "AgentState",
    "AgentStateToolRulesItem",
    "AgentType",
    "AgentsSearchRequestCombinator",
    "AgentsSearchRequestSearchItem",
    "AgentsSearchRequestSearchItemDirection",
    "AgentsSearchRequestSearchItemDirectionDirection",
    "AgentsSearchRequestSearchItemDirectionValue",
    "AgentsSearchRequestSearchItemOne",
    "AgentsSearchRequestSearchItemOneOperator",
    "AgentsSearchRequestSearchItemTwo",
    "AgentsSearchRequestSearchItemZero",
    "AppAuthScheme",
    "AppAuthSchemeAuthMode",
    "AppModel",
    "AssistantMessage",
    "AssistantMessageContent",
    "AsyncLetta",
    "AuthRequest",
    "AuthResponse",
    "AuthSchemeField",
    "Block",
    "BlockUpdate",
    "ChatCompletionMessageToolCall",
    "ChildToolRule",
    "ConditionalToolRule",
    "ConflictError",
    "ConflictErrorBody",
    "ContextWindowOverview",
    "CreateAgentRequestToolRulesItem",
    "CreateBlock",
    "E2BSandboxConfig",
    "EmbeddingConfig",
    "EmbeddingConfigEmbeddingEndpointType",
    "FileMetadata",
    "Function",
    "FunctionDefinition",
    "FunctionTool",
    "Health",
    "HttpValidationError",
    "InitToolRule",
    "InternalServerError",
    "InternalServerErrorBody",
    "Job",
    "JobStatus",
    "JobType",
    "Letta",
    "LettaEnvironment",
    "LettaMessageUnion",
    "LettaRequest",
    "LettaRequestConfig",
    "LettaResponse",
    "LettaUsageStatistics",
    "LlmConfig",
    "LlmConfigModelEndpointType",
    "LocalSandboxConfig",
    "Memory",
    "Message",
    "MessageCreate",
    "MessageCreateContent",
    "MessageCreateRole",
    "MessageRole",
    "NotFoundError",
    "NotFoundErrorBody",
    "NotFoundErrorBodyMessage",
    "Organization",
    "OrganizationCreate",
    "Passage",
    "Provider",
    "ReasoningMessage",
    "Run",
    "SandboxConfig",
    "SandboxConfigCreate",
    "SandboxConfigCreateConfig",
    "SandboxConfigUpdate",
    "SandboxConfigUpdateConfig",
    "SandboxEnvironmentVariable",
    "SandboxEnvironmentVariableCreate",
    "SandboxEnvironmentVariableUpdate",
    "SandboxType",
    "Source",
    "SystemMessage",
    "SystemMessageContent",
    "TerminalToolRule",
    "TextContent",
    "Tool",
    "ToolCall",
    "ToolCallDelta",
    "ToolCallMessage",
    "ToolCallMessageToolCall",
    "ToolCreate",
    "ToolReturnMessage",
    "ToolReturnMessageStatus",
    "ToolRuleType",
    "ToolType",
    "UnprocessableEntityError",
    "UpdateAgentToolRulesItem",
    "UsageStatistics",
    "User",
    "UserCreate",
    "UserMessage",
    "UserMessageContent",
    "UserUpdate",
    "ValidationError",
    "ValidationErrorLocItem",
    "__version__",
    "agents",
    "blocks",
    "health",
    "jobs",
    "models",
    "providers",
    "runs",
    "sources",
    "tag",
    "tools",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...core.request_options import RequestOptions
from .types.messages_list_response import MessagesListResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.message_create import MessageCreate
from ...types.letta_request_config import LettaRequestConfig
from ...types.letta_response import LettaResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...types.message_role import MessageRole
from .types.message_update_content import MessageUpdateContent
from ...types.chat_completion_message_tool_call import ChatCompletionMessageToolCall
from ...types.message import Message
from .types.letta_streaming_response import LettaStreamingResponse
import httpx_sse
import json
from ...types.run import Run
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MessagesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        agent_id: str,
        *,
        before: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        msg_object: typing.Optional[bool] = None,
        assistant_message_tool_name: typing.Optional[str] = None,
        assistant_message_tool_kwarg: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MessagesListResponse:
        """
        Retrieve message history for an agent.

        Parameters
        ----------
        agent_id : str

        before : typing.Optional[str]
            Message before which to retrieve the returned messages.

        limit : typing.Optional[int]
            Maximum number of messages to retrieve.

        msg_object : typing.Optional[bool]
            If true, returns Message objects. If false, return LettaMessage objects.

        assistant_message_tool_name : typing.Optional[str]
            The name of the designated message tool.

        assistant_message_tool_kwarg : typing.Optional[str]
            The name of the message argument in the designated message tool.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MessagesListResponse
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.agents.messages.list(
            agent_id="agent_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages",
            method="GET",
            params={
                "before": before,
                "limit": limit,
                "msg_object": msg_object,
                "assistant_message_tool_name": assistant_message_tool_name,
                "assistant_message_tool_kwarg": assistant_message_tool_kwarg,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    MessagesListResponse,
                    construct_type(
                        type_=MessagesListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        agent_id: str,
        *,
        messages: typing.Sequence[MessageCreate],
        config: typing.Optional[LettaRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LettaResponse:
        """
        Process a user message and return the agent's response.
        This endpoint accepts a message from a user and processes it through the agent.

        Parameters
        ----------
        agent_id : str

        messages : typing.Sequence[MessageCreate]
            The messages to be sent to the agent.

        config : typing.Optional[LettaRequestConfig]
            Configuration options for the LettaRequest.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LettaResponse
            Successful Response

        Examples
        --------
        from letta_client import Letta, MessageCreate

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.agents.messages.create(
            agent_id="agent_id",
            messages=[
                MessageCreate(
                    role="user",
                    content="content",
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages",
            method="POST",
            json={
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=typing.Sequence[MessageCreate], direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=LettaRequestConfig, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    LettaResponse,
                    construct_type(
                        type_=LettaResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def modify(
        self,
        agent_id: str,
        message_id: str,
        *,
        role: typing.Optional[MessageRole] = OMIT,
        content: typing.Optional[MessageUpdateContent] = OMIT,
        name: typing.Optional[str] = OMIT,
        tool_calls: typing.Optional[typing.Sequence[ChatCompletionMessageToolCall]] = OMIT,
        tool_call_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Message:
        """
        Update the details of a message associated with an agent.

        Parameters
        ----------
        agent_id : str

        message_id : str

        role : typing.Optional[MessageRole]
            The role of the participant.

        content : typing.Optional[MessageUpdateContent]
            The content of the message.

        name : typing.Optional[str]
            The name of the participant.

        tool_calls : typing.Optional[typing.Sequence[ChatCompletionMessageToolCall]]
            The list of tool calls requested.

        tool_call_id : typing.Optional[str]
            The id of the tool call.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Message
            Successful Response

        Examples
        --------
        from letta_client import Letta

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.agents.messages.modify(
            agent_id="agent_id",
            message_id="message_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages/{jsonable_encoder(message_id)}",
            method="PATCH",
            json={
                "role": role,
                "content": convert_and_respect_annotation_metadata(
                    object_=content, annotation=MessageUpdateContent, direction="write"
                ),
                "name": name,
                "tool_calls": convert_and_respect_annotation_metadata(
                    object_=tool_calls, annotation=typing.Sequence[ChatCompletionMessageToolCall], direction="write"
                ),
                "tool_call_id": tool_call_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Message,
                    construct_type(
                        type_=Message,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_stream(
        self,
        agent_id: str,
        *,
        messages: typing.Sequence[MessageCreate],
        config: typing.Optional[LettaRequestConfig] = OMIT,
        stream_tokens: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[LettaStreamingResponse]:
        """
        Process a user message and return the agent's response.
        This endpoint accepts a message from a user and processes it through the agent.
        It will stream the steps of the response always, and stream the tokens if 'stream_tokens' is set to True.

        Parameters
        ----------
        agent_id : str

        messages : typing.Sequence[MessageCreate]
            The messages to be sent to the agent.

        config : typing.Optional[LettaRequestConfig]
            Configuration options for the LettaRequest.

        stream_tokens : typing.Optional[bool]
            Flag to determine if individual tokens should be streamed. Set to True for token streaming (requires stream_steps = True).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[LettaStreamingResponse]
            Successful response

        Examples
        --------
        from letta_client import Letta, MessageCreate

        client = Letta(
            token="YOUR_TOKEN",
        )
        response = client.agents.messages.create_stream(
            agent_id="agent_id",
            messages=[
                MessageCreate(
                    role="user",
                    content="content",
                )
            ],
        )
        for chunk in response:
            yield chunk
        """
        with self._client_wrapper.httpx_client.stream(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages/stream",
            method="POST",
            json={
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=typing.Sequence[MessageCreate], direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=LettaRequestConfig, direction="write"
                ),
                "stream_tokens": stream_tokens,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    _event_source = httpx_sse.EventSource(_response)
                    for _sse in _event_source.iter_sse():
                        try:
                            yield typing.cast(
                                LettaStreamingResponse,
                                construct_type(
                                    type_=LettaStreamingResponse,  # type: ignore
                                    object_=json.loads(_sse.data),
                                ),
                            )
                        except:
                            pass
                    return
                _response.read()
                if _response.status_code == 422:
                    raise UnprocessableEntityError(
                        typing.cast(
                            HttpValidationError,
                            construct_type(
                                type_=HttpValidationError,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_async(
        self,
        agent_id: str,
        *,
        messages: typing.Sequence[MessageCreate],
        config: typing.Optional[LettaRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Run:
        """
        Asynchronously process a user message and return a run object.
        The actual processing happens in the background, and the status can be checked using the run ID.

        Parameters
        ----------
        agent_id : str

        messages : typing.Sequence[MessageCreate]
            The messages to be sent to the agent.

        config : typing.Optional[LettaRequestConfig]
            Configuration options for the LettaRequest.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Run
            Successful Response

        Examples
        --------
        from letta_client import Letta, MessageCreate

        client = Letta(
            token="YOUR_TOKEN",
        )
        client.agents.messages.create_async(
            agent_id="agent_id",
            messages=[
                MessageCreate(
                    role="user",
                    content="content",
                )
            ],
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages/async",
            method="POST",
            json={
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=typing.Sequence[MessageCreate], direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=LettaRequestConfig, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Run,
                    construct_type(
                        type_=Run,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncMessagesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        agent_id: str,
        *,
        before: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        msg_object: typing.Optional[bool] = None,
        assistant_message_tool_name: typing.Optional[str] = None,
        assistant_message_tool_kwarg: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MessagesListResponse:
        """
        Retrieve message history for an agent.

        Parameters
        ----------
        agent_id : str

        before : typing.Optional[str]
            Message before which to retrieve the returned messages.

        limit : typing.Optional[int]
            Maximum number of messages to retrieve.

        msg_object : typing.Optional[bool]
            If true, returns Message objects. If false, return LettaMessage objects.

        assistant_message_tool_name : typing.Optional[str]
            The name of the designated message tool.

        assistant_message_tool_kwarg : typing.Optional[str]
            The name of the message argument in the designated message tool.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MessagesListResponse
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.messages.list(
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages",
            method="GET",
            params={
                "before": before,
                "limit": limit,
                "msg_object": msg_object,
                "assistant_message_tool_name": assistant_message_tool_name,
                "assistant_message_tool_kwarg": assistant_message_tool_kwarg,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    MessagesListResponse,
                    construct_type(
                        type_=MessagesListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        agent_id: str,
        *,
        messages: typing.Sequence[MessageCreate],
        config: typing.Optional[LettaRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LettaResponse:
        """
        Process a user message and return the agent's response.
        This endpoint accepts a message from a user and processes it through the agent.

        Parameters
        ----------
        agent_id : str

        messages : typing.Sequence[MessageCreate]
            The messages to be sent to the agent.

        config : typing.Optional[LettaRequestConfig]
            Configuration options for the LettaRequest.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        LettaResponse
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta, MessageCreate

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.messages.create(
                agent_id="agent_id",
                messages=[
                    MessageCreate(
                        role="user",
                        content="content",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages",
            method="POST",
            json={
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=typing.Sequence[MessageCreate], direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=LettaRequestConfig, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    LettaResponse,
                    construct_type(
                        type_=LettaResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def modify(
        self,
        agent_id: str,
        message_id: str,
        *,
        role: typing.Optional[MessageRole] = OMIT,
        content: typing.Optional[MessageUpdateContent] = OMIT,
        name: typing.Optional[str] = OMIT,
        tool_calls: typing.Optional[typing.Sequence[ChatCompletionMessageToolCall]] = OMIT,
        tool_call_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Message:
        """
        Update the details of a message associated with an agent.

        Parameters
        ----------
        agent_id : str

        message_id : str

        role : typing.Optional[MessageRole]
            The role of the participant.

        content : typing.Optional[MessageUpdateContent]
            The content of the message.

        name : typing.Optional[str]
            The name of the participant.

        tool_calls : typing.Optional[typing.Sequence[ChatCompletionMessageToolCall]]
            The list of tool calls requested.

        tool_call_id : typing.Optional[str]
            The id of the tool call.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Message
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.messages.modify(
                agent_id="agent_id",
                message_id="message_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages/{jsonable_encoder(message_id)}",
            method="PATCH",
            json={
                "role": role,
                "content": convert_and_respect_annotation_metadata(
                    object_=content, annotation=MessageUpdateContent, direction="write"
                ),
                "name": name,
                "tool_calls": convert_and_respect_annotation_metadata(
                    object_=tool_calls, annotation=typing.Sequence[ChatCompletionMessageToolCall], direction="write"
                ),
                "tool_call_id": tool_call_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Message,
                    construct_type(
                        type_=Message,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_stream(
        self,
        agent_id: str,
        *,
        messages: typing.Sequence[MessageCreate],
        config: typing.Optional[LettaRequestConfig] = OMIT,
        stream_tokens: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[LettaStreamingResponse]:
        """
        Process a user message and return the agent's response.
        This endpoint accepts a message from a user and processes it through the agent.
        It will stream the steps of the response always, and stream the tokens if 'stream_tokens' is set to True.

        Parameters
        ----------
        agent_id : str

        messages : typing.Sequence[MessageCreate]
            The messages to be sent to the agent.

        config : typing.Optional[LettaRequestConfig]
            Configuration options for the LettaRequest.

        stream_tokens : typing.Optional[bool]
            Flag to determine if individual tokens should be streamed. Set to True for token streaming (requires stream_steps = True).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[LettaStreamingResponse]
            Successful response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta, MessageCreate

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.agents.messages.create_stream(
                agent_id="agent_id",
                messages=[
                    MessageCreate(
                        role="user",
                        content="content",
                    )
                ],
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._client_wrapper.httpx_client.stream(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages/stream",
            method="POST",
            json={
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=typing.Sequence[MessageCreate], direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=LettaRequestConfig, direction="write"
                ),
                "stream_tokens": stream_tokens,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        ) as _response:
            try:
                if 200 <= _response.status_code < 300:
                    _event_source = httpx_sse.EventSource(_response)
                    async for _sse in _event_source.aiter_sse():
                        try:
                            yield typing.cast(
                                LettaStreamingResponse,
                                construct_type(
                                    type_=LettaStreamingResponse,  # type: ignore
                                    object_=json.loads(_sse.data),
                                ),
                            )
                        except:
                            pass
                    return
                await _response.aread()
                if _response.status_code == 422:
                    raise UnprocessableEntityError(
                        typing.cast(
                            HttpValidationError,
                            construct_type(
                                type_=HttpValidationError,  # type: ignore
                                object_=_response.json(),
                            ),
                        )
                    )
                _response_json = _response.json()
            except JSONDecodeError:
                raise ApiError(status_code=_response.status_code, body=_response.text)
            raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_async(
        self,
        agent_id: str,
        *,
        messages: typing.Sequence[MessageCreate],
        config: typing.Optional[LettaRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Run:
        """
        Asynchronously process a user message and return a run object.
        The actual processing happens in the background, and the status can be checked using the run ID.

        Parameters
        ----------
        agent_id : str

        messages : typing.Sequence[MessageCreate]
            The messages to be sent to the agent.

        config : typing.Optional[LettaRequestConfig]
            Configuration options for the LettaRequest.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Run
            Successful Response

        Examples
        --------
        import asyncio

        from letta_client import AsyncLetta, MessageCreate

        client = AsyncLetta(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.agents.messages.create_async(
                agent_id="agent_id",
                messages=[
                    MessageCreate(
                        role="user",
                        content="content",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/agents/{jsonable_encoder(agent_id)}/messages/async",
            method="POST",
            json={
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=typing.Sequence[MessageCreate], direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=LettaRequestConfig, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    Run,
                    construct_type(
                        type_=Run,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        construct_type(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
